# Configuration file for the Sphinx documentation builder.
#
# https://www.sphinx-doc.org/en/master/usage/configuration.html

import sys
from pathlib import Path

if sys.version_info < (3, 11):
    import tomli as tomllib
else:
    import tomllib

DIR = Path(__file__).parent.resolve()

# -- Project information ----------------------------------------------------

project = 'CMake Python Distributions'
copyright = '2016, Jean-Christophe Fillion-Robin and contributors'
author = 'Jean-Christophe Fillion-Robin'

# The full version, including alpha/beta/rc tags, read from pyproject.toml.
with (DIR.parent / "pyproject.toml").open('rb') as f:
    release = tomllib.load(f)["project"]["version"]

# A shorter version
version = ".".join(release.split(".")[:2])

# -- General configuration --------------------------------------------------

extensions = ['sphinxcontrib.mermaid']

language = 'en'

exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# -- Options for HTML output ------------------------------------------------

html_theme = 'furo'

html_theme_options = {
    "source_repository": "https://github.com/scikit-build/cmake-python-distributions",
    "source_branch": "main",
    "source_directory": "docs/",
    "footer_icons": [
        {
            "name": "GitHub",
            "url": "https://github.com/scikit-build/cmake-python-distributions",
            "html": """
                <svg stroke="currentColor" fill="currentColor" stroke-width="0" viewBox="0 0 16 16">
                    <path fill-rule="evenodd" d="M8 0C3.58 0 0 3.58 0 8c0 3.54 2.29 6.53 5.47 7.59.4.07.55-.17.55-.38 0-.19-.01-.82-.01-1.49-2.01.37-2.53-.49-2.69-.94-.09-.23-.48-.94-.82-1.13-.28-.15-.68-.52-.01-.53.63-.01 1.08.58 1.23.82.72 1.21 1.87.87 2.33.66.07-.52.28-.87.51-1.07-1.78-.2-3.64-.89-3.64-3.95 0-.87.31-1.59.82-2.15-.08-.2-.36-1.02.08-2.12 0 0 .67-.21 2.2.82.64-.18 1.32-.27 2-.27.68 0 1.36.09 2 .27 1.53-1.04 2.2-.82 2.2-.82.44 1.1.16 1.92.08 2.12.51.56.82 1.27.82 2.15 0 3.07-1.87 3.75-3.65 3.95.29.25.54.73.54 1.48 0 1.07-.01 1.93-.01 2.2 0 .21.15.46.55.38A8.013 8.013 0 0 0 16 8c0-4.42-3.58-8-8-8z"></path>
                </svg>
            """,
            "class": "",
        },
    ],
}
html_copy_source = False
html_show_sourcelink = False



#!/usr/bin/env python3
#
# /// script
# dependencies = ["requests"]
# ///

"""
Command line executable allowing to update OpenSSL version.
"""

from __future__ import annotations

import argparse
import contextlib
import re
import textwrap
from pathlib import Path

import requests

TYPE_CHECKING = False

if TYPE_CHECKING:
    from collections.abc import Generator

ROOT_DIR = Path(__file__).parent.parent.resolve()


@contextlib.contextmanager
def _log(txt: str, verbose: bool=True) -> Generator[None, None, None]:
    if verbose:
        print(txt)
    yield
    if verbose:
        print(txt, "-", "done")


def get_openssl_sha256(version: str, verbose: bool=False) -> str:
    files_base_url = (
        f"https://github.com/openssl/openssl/releases/download/openssl-{version}/openssl-{version}.tar.gz.sha256"
    )
    with _log(f"Collecting SHA256 from '{files_base_url}'"):
        parts = requests.get(files_base_url).content.decode("ascii").strip().split()
        sha256 = parts[0]
        if len(parts) > 1:
            expected_parts = 2  #  f"{sha256} {filename}""
            assert len(parts) == expected_parts
            assert parts[1].lstrip("*") == f"openssl-{version}.tar.gz"
        if verbose:
            print("got sha256:", sha256)
        return sha256


def _update_file(filepath: Path, regex: re.Pattern[str], replacement: str) -> None:
    with _log(f"Updating {filepath.relative_to(ROOT_DIR)}"):
        pattern = re.compile(regex)
        with filepath.open() as doc_file:
            updated_content = [pattern.sub(replacement, line) for line in doc_file]
        with filepath.open("w") as doc_file:
            doc_file.writelines(updated_content)


def update_openssl_script(version: str, sha256: str) -> None:
    pattern = re.compile(r"^OPENSSL_ROOT=.*")
    replacement = f"OPENSSL_ROOT=openssl-{version}"
    _update_file(
        ROOT_DIR / "scripts/manylinux-build-and-install-openssl.sh", pattern, replacement
    )
    pattern = re.compile(r"^OPENSSL_HASH=.*")
    replacement = f"OPENSSL_HASH={sha256}"
    _update_file(
        ROOT_DIR / "scripts/manylinux-build-and-install-openssl.sh", pattern, replacement
    )


def main() -> None:
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "openssl_version",
        metavar="OPENSSL_VERSION",
        type=str,
        help="OpenSSL version",
    )
    parser.add_argument(
        "--collect-only",
        action="store_true",
        help="If specified, only display the hashsum for the requested version",
    )
    parser.add_argument(
        "--quiet",
        action="store_true",
        help="Hide the output",
    )
    args = parser.parse_args()

    sha256 = get_openssl_sha256(args.openssl_version, verbose=args.collect_only)
    if args.collect_only:
        return

    update_openssl_script(args.openssl_version, sha256)

    if not args.quiet:
        msg = """\
            Complete! Now run:

            git switch -c update-to-openssl-{release}
            git add -u scripts/manylinux-build-and-install-openssl.sh
            git commit -m "Update to OpenSSL {release}"
            gh pr create --fill --body "Created by update_openssl_version.py"
            """
        print(textwrap.dedent(msg.format(release=args.openssl_version)))


if __name__ == "__main__":
    main()



import os
import subprocess
import sys
import sysconfig
import textwrap
from importlib.metadata import distribution

import pytest

import cmake

from . import push_argv

all_tools = pytest.mark.parametrize("tool", ["cmake", "cpack", "ctest"])


def _run(program, args):
    func = getattr(cmake, program)
    args = [f"{program}.py", *args]
    with push_argv(args), pytest.raises(SystemExit) as excinfo:
        func()
    assert excinfo.value.code == 0


@all_tools
def test_cmake_module(tool, monkeypatch):
    monkeypatch.setattr(sys, "platform", "win32")  # do not use os.execl
    _run(tool, ["--version"])


def test_cmake_https(tmpdir, monkeypatch):
    monkeypatch.setattr(sys, "platform", "win32")  # do not use os.execl
    test_script = tmpdir.join("cmake-test-https-download.cmake")
    test_script.write(textwrap.dedent(
        """
        file(
          DOWNLOAD
            https://github.com/scikit-build/cmake-python-distributions
            ${TMP_DIR}/page.html
          SHOW_PROGRESS
          STATUS status
          )
        list(GET status 0 error_code)
        list(GET status 1 error_msg)
        if(error_code)
            message(
                FATAL_ERROR "error: Failed to download ${url}: ${error_msg}")
        endif()
        """
    ))

    _run("cmake", [f"-DTMP_DIR:PATH={tmpdir}", "-P", str(test_script)])


def _get_scripts():
    dist = distribution("cmake")
    scripts_paths = [os.path.abspath(sysconfig.get_path("scripts", scheme)) for scheme in sysconfig.get_scheme_names()]
    scripts = []
    for file in dist.files:
        if os.path.abspath(str(file.locate().parent)) in scripts_paths:
            scripts.append(file.locate().resolve(strict=True))
    return scripts


@all_tools
def test_cmake_script(tool):
    expected_version = "4.4.0"
    scripts = [script for script in _get_scripts() if script.stem == tool]
    assert len(scripts) == 1
    output = subprocess.check_output([str(scripts[0]), "--version"]).decode("ascii")
    assert output.splitlines()[0] == f"{tool} version {expected_version}"


def test_cmake_main():
    expected_version = "4.4.0"
    output = subprocess.run([sys.executable, "-m", "cmake", "--version"], text=True, capture_output=True, check=False).stdout
    assert output.splitlines()[0] == f"cmake version {expected_version}"



from __future__ import annotations

import os

from scikit_build_core import build as _orig

if hasattr(_orig, "prepare_metadata_for_build_editable"):
    prepare_metadata_for_build_editable = _orig.prepare_metadata_for_build_editable
if hasattr(_orig, "prepare_metadata_for_build_wheel"):
    prepare_metadata_for_build_wheel = _orig.prepare_metadata_for_build_wheel
build_editable = _orig.build_editable
build_sdist = _orig.build_sdist
get_requires_for_build_editable = _orig.get_requires_for_build_editable
get_requires_for_build_sdist = _orig.get_requires_for_build_sdist


def _strtobool(value: str) -> bool:
    """
    Converts a environment variable string into a boolean value.
    """
    if not value:
        return False
    value = value.lower()
    if value.isdigit():
        return bool(int(value))
    return value not in {"n", "no", "off", "false", "f"}


def get_requires_for_build_wheel(
    config_settings: dict[str, str | list[str]] | None = None,
) -> list[str]:
    packages_orig = _orig.get_requires_for_build_wheel(config_settings)
    allow_cmake = _strtobool(os.environ.get("CMAKE_PYTHON_DIST_ALLOW_CMAKE_DEP", ""))
    allow_ninja = any(
        _strtobool(os.environ.get(var, ""))
        for var in ("CMAKE_PYTHON_DIST_FORCE_NINJA_DEP", "CMAKE_PYTHON_DIST_ALLOW_NINJA_DEP")
    )
    packages = []
    for package in packages_orig:
        package_name = package.lower().split(">")[0].strip()
        if package_name == "cmake" and not allow_cmake:
            continue
        if package_name == "ninja" and not allow_ninja:
            continue
        packages.append(package)
    return packages


def _bootstrap_build(temp_path: str, config_settings: dict[str, list[str] | str] | None = None) -> str:
    import hashlib
    import platform
    import re
    import shutil
    import subprocess
    import tarfile
    import urllib.request
    import zipfile
    from pathlib import Path

    env = os.environ.copy()
    temp_path_ = Path(temp_path)

    archive_dir = temp_path_
    if config_settings:
        archive_dir = Path(config_settings.get("cmake.define.CMakePythonDistributions_ARCHIVE_DOWNLOAD_DIR", archive_dir))
        archive_dir.mkdir(parents=True, exist_ok=True)

    if os.name == "posix":
        if "MAKE" not in env:
            make_path = None
            make_candidates = ("gmake", "make", "smake")
            for candidate in make_candidates:
                make_path = shutil.which(candidate)
                if make_path is not None:
                    break
            if make_path is None:
                msg = f"Could not find a make program. Tried {make_candidates!r}"
                raise ValueError(msg)
            env["MAKE"] = make_path
        make_path = env["MAKE"]
        kind = "unix_source"
    else:
        assert os.name == "nt"
        machine = platform.machine()
        kinds = {
            "x86": "win32_binary",
            "AMD64": "win64_binary",
            "ARM64": "winarm64_binary",
        }
        if machine not in kinds:
            msg = f"Could not find CMake required to build on a {machine} system"
            raise ValueError(msg)
        kind = kinds[machine]


    cmake_urls = Path("CMakeUrls.cmake").read_text()
    archive_url = re.findall(rf'set\({kind}_url\s+"(?P<data>.*)"\)$', cmake_urls, flags=re.MULTILINE)[0]
    archive_sha256 = re.findall(rf'set\({kind}_sha256\s+"(?P<data>.*)"\)$', cmake_urls, flags=re.MULTILINE)[0]

    archive_name = archive_url.rsplit("/", maxsplit=1)[1]
    archive_path = archive_dir / archive_name
    if not archive_path.exists():
        with urllib.request.urlopen(archive_url) as response:
            archive_path.write_bytes(response.read())

    sha256 = hashlib.sha256(archive_path.read_bytes()).hexdigest()
    if archive_sha256.lower() != sha256.lower():
        msg = f"Invalid sha256 for {archive_url!r}. Expected {archive_sha256!r}, got {sha256!r}"
        raise ValueError(msg)

    if os.name == "posix":
        assert archive_name.endswith(".tar.gz")
        tar_filter_kwargs = {"filter": "tar"} if hasattr(tarfile, "tar_filter") else {}
        with tarfile.open(archive_path) as tar:
            tar.extractall(path=temp_path_, **tar_filter_kwargs)

        parallel_str = env.get("CMAKE_BUILD_PARALLEL_LEVEL", "1")
        parallel = max(0, int(parallel_str) if parallel_str.isdigit() else 1) or os.cpu_count() or 1

        bootstrap_path = next(temp_path_.glob("cmake-*/bootstrap"))
        prefix_path = temp_path_ / "cmake-install"
        cmake_path = prefix_path / "bin" / "cmake"
        bootstrap_args = [f"--prefix={prefix_path}", "--no-qt-gui", "--no-debugger", f"--parallel={parallel}", "--", "-DBUILD_TESTING=OFF", "-DBUILD_CursesDialog:BOOL=OFF"]
        previous_cwd = Path().absolute()
        os.chdir(bootstrap_path.parent)
        try:
            subprocess.run([bootstrap_path, *bootstrap_args], env=env, check=True)
            subprocess.run([make_path, "-j", f"{parallel}"], env=env, check=True)
            subprocess.run([make_path, "install"], env=env, check=True)
        finally:
            os.chdir(previous_cwd)
    else:
        assert archive_name.endswith(".zip")
        with zipfile.ZipFile(archive_path) as zip_:
            zip_.extractall(path=temp_path_)
        cmake_path = next(temp_path_.glob("cmake-*/bin/cmake.exe"))

    return str(cmake_path)


def build_wheel(
    wheel_directory: str,
    config_settings: dict[str, list[str] | str] | None = None,
    metadata_directory: str | None = None,
) -> str:
    from scikit_build_core.errors import CMakeNotFoundError

    try:
        return _orig.build_wheel(wheel_directory, config_settings, metadata_directory)
    except CMakeNotFoundError:
        if os.name not in {"posix", "nt"}:
            raise
    # Let's try bootstrapping CMake
    import tempfile
    with tempfile.TemporaryDirectory() as temp_path:
        cmake_path = _bootstrap_build(temp_path, config_settings)
        assert cmake_path
        os.environ["CMAKE_EXECUTABLE"] = cmake_path
        return _orig.build_wheel(wheel_directory, config_settings, metadata_directory)



# /// script
# dependencies = ["nox>=2025.2.9"]
# ///

import argparse
import re
from pathlib import Path

import nox

nox.needs_version = ">=2025.2.9"
nox.options.default_venv_backend = "uv|virtualenv"

BUILD_ENV = {
    "MACOSX_DEPLOYMENT_TARGET": "10.10",
    "ARCHFLAGS": "-arch x86_64 -arch arm64",
}

wheel = ""


@nox.session
def build(session: nox.Session) -> str:
    """
    Make an SDist and a wheel.
    """
    session.log(
        "The files produced locally by this job are not intended to be redistributable"
    )
    extra = ["--installer=uv"] if session.venv_backend == "uv" else []
    session.install("build")
    tmpdir = session.create_tmp()
    session.run("python", "-m", "build", "--outdir", tmpdir, *extra, env=BUILD_ENV)
    (wheel_path,) = Path(tmpdir).glob("*.whl")
    (sdist_path,) = Path(tmpdir).glob("*.tar.gz")
    Path("dist").mkdir(exist_ok=True)
    wheel_path.rename(f"dist/{wheel_path.name}")
    sdist_path.rename(f"dist/{sdist_path.name}")

    global wheel
    wheel = f"dist/{wheel_path.name}"


@nox.session
def lint(session: nox.Session) -> str:
    """
    Run linters on the codebase.
    """
    session.install("pre-commit")
    session.run("pre-commit", "run", "-a")


@nox.session(requires=["build"])
def tests(session: nox.Session) -> str:
    """
    Run the tests.
    """
    pyproject = nox.project.load_toml("pyproject.toml")
    deps = nox.project.dependency_groups(pyproject, "test")
    session.install(wheel, *deps)
    session.run("pytest", *session.posargs)


@nox.session(reuse_venv=True, default=False)
def docs(session: nox.Session) -> None:
    """
    Build the docs. Pass "--non-interactive" to avoid serve. Pass "-- -b linkcheck" to check links.
    """
    pyproject = nox.project.load_toml("pyproject.toml")
    deps = nox.project.dependency_groups(pyproject, "docs")

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-b", dest="builder", default="html", help="Build target (default: html)"
    )
    args, posargs = parser.parse_known_args(session.posargs)
    serve = args.builder == "html" and session.interactive

    extra_installs = ["sphinx-autobuild"] if serve else []
    session.install(*deps, *extra_installs)
    session.chdir("docs")

    if args.builder == "linkcheck":
        session.run(
            "sphinx-build", "-b", "linkcheck", ".", "_build/linkcheck", *posargs
        )
        return

    shared_args = (
        "-n",  # nitpicky mode
        "-T",  # full tracebacks
        f"-b={args.builder}",
        ".",
        f"_build/{args.builder}",
        *posargs,
    )

    if serve:
        session.run(
            "sphinx-autobuild", "--open-browser", "--ignore=.build", *shared_args
        )
    else:
        session.run("sphinx-build", "--keep-going", *shared_args)

def _bump(session: nox.Session, name: str, repository: str, branch: str, script: str, files) -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--commit", action="store_true", help="Make a branch and commit."
    )
    parser.add_argument(
        "version", nargs="?", help="The version to process - leave off for latest."
    )
    args = parser.parse_args(session.posargs)

    if args.version is None:
        session.install("lastversion", "requests")
        lastversion_args = []
        if branch:
            lastversion_args.extend(("--branch", branch, "--only", f"{branch}."))
        lastversion_args.append(repository)
        version = session.run("lastversion", *lastversion_args, log=False, silent=True).strip()
    else:
        session.install("requests")
        version = args.version

    extra = ["--quiet"] if args.commit else []
    session.run("python", script, version, *extra)

    if args.commit:
        session.run("git", "switch", "-c", f"update-to-{name.lower()}-{version}", external=True)
        session.run("git", "add", "-u", *files, external=True)
        session.run("git", "commit", "-m", f"Update to {name} {version}", external=True)
        session.log(
            f'Complete! Now run: gh pr create --fill --body "Created by running `nox -s {session.name} -- --commit`"'
        )


@nox.session(default=False)
def bump(session: nox.Session) -> None:
    """
    Set to a new version, use -- <version>, otherwise will use the latest version.
    """
    files = (
        "pyproject.toml",
        "CMakeUrls.cmake",
        "docs/index.rst",
        "README.rst",
        "tests/test_cmake.py",
        "docs/update_cmake_version.rst",
        "docs/make_a_release.rst",
    )
    _bump(session, "CMake", "kitware/cmake", "", "scripts/update_cmake_version.py", files)


@nox.session(name="bump-openssl", default=False)
def bump_openssl(session: nox.Session) -> None:
    """
    Set openssl to a new version, use -- <version>, otherwise will use the latest version.
    """
    files = (
        "scripts/manylinux-build-and-install-openssl.sh",
    )
    _bump(session, "OpenSSL", "openssl/openssl", "3.5", "scripts/update_openssl_version.py", files)


def _get_version() -> str:
    txt = Path("pyproject.toml").read_text()
    return next(iter(re.finditer(r'^version = "([\d\.]+)"$', txt, flags=re.MULTILINE))).group(1)


@nox.session(venv_backend="none", default=False)
def tag_release(session: nox.Session) -> None:
    """
    Print instructions for tagging a release and pushing it to GitHub.
    """

    session.log("Run the following commands to make a release:")
    current_version = _get_version()
    print(f"git tag --sign -m 'cmake-python-distributions {current_version}' {current_version} main")
    print(f"git push origin {current_version}")


@nox.session(venv_backend="none", default=False)
def cmake_version(session: nox.Session) -> None:  # noqa: ARG001
    """
    Print upstream cmake version.
    """

    current_version = _get_version()
    print(".".join(current_version.split(".")[:3]))


@nox.session(venv_backend="none", default=False)
def openssl_version(session: nox.Session) -> None:  # noqa: ARG001
    """
    Print upstream OpenSSL version.
    """
    txt = Path("scripts/manylinux-build-and-install-openssl.sh").read_text()
    current_version = next(iter(re.finditer(r'^OPENSSL_ROOT=openssl-([\d\.]+)$', txt, flags=re.MULTILINE))).group(1)
    print(".".join(current_version.split(".")[:3]))


if __name__ == "__main__":
    nox.main()
